package com.newxtc.fw.client.util;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CookieUtil {

	private final static Logger logger = LoggerFactory.getLogger(CookieUtil.class);

	public void setCookie(HttpServletResponse response, String domain, int expireTime, String name, String value) {
		// 如果value不为空，写入cookie
		if (StringUtils.isNotBlank(value)) {
			Cookie cookie = new Cookie(name, value);
			cookie.setPath("/");
			if (domain != null && !domain.isEmpty())
				cookie.setDomain(domain);
			cookie.setMaxAge(expireTime);
			response.addCookie(cookie);
		}
	}

	public void setCookie(HttpServletResponse response, String name, String value) {
		// 如果value不为空，写入cookie
		if (StringUtils.isNotBlank(value)) {
			Cookie cookie = new Cookie(name, value);
			cookie.setPath("/");
			response.addCookie(cookie);
		}
	}

	public String getCookie(HttpServletRequest request, String name) {
		if (name == null || "".equals(name)) {
			logger.error("getCookie() name=" + name);
			return null;
		}
		try {
			Map<String, String> cookieMap = new HashMap<String, String>();
			StringBuffer sb = new StringBuffer();
			Cookie[] cookieArray = request.getCookies();
			if (cookieArray != null && cookieArray.length > 0) {
				String cookieName, cookieValue;
				int cookLen;
				for (Cookie cookie : cookieArray) {
					cookieName = cookie.getName();
					if ("JSESSIONID".equals(cookieName.toUpperCase())) {
						continue;
					}
					cookieValue = cookie.getValue();
					if (cookieName != null && cookieValue != null) {
						cookieMap.put(cookieName.trim(), cookieValue.trim());
						cookLen = cookieValue.length();
						if (sb.length() > 0)
							sb.append(",");
						sb.append(cookieName + "=" + cookLen);
					}
				}
			}
			String value = cookieMap.get(name);
			if (value != null) {
				return value;
			} else {
				if (sb.length() > 0) {
					logger.debug("getCookie() " + name + "=" + value + ",{" + sb.toString() + "}");
				}
				return null;
			}
		} catch (Exception e) {
			logger.error("getCookie() name=" + name + ",e=" + e.toString());
			return null;
		}
	}

	public void removeCookie(HttpServletRequest request, HttpServletResponse response, String domain, String name) {
		try {
			Cookie cookie = new Cookie(name, null);
			if (domain != null)
				cookie.setDomain(domain);
			cookie.setPath("/");
			cookie.setMaxAge(0);
			response.addCookie(cookie);
			logger.debug("removeCookie cookie=" + cookieToString(cookie));
		} catch (Exception e) {
			logger.error("removeCookie() error=" + e);
		}
	}

	private String cookieToString(Cookie cookie) {
		if (cookie == null)
			return "null";
		return "Cookie[name=" + cookie.getName() + ", value=" + cookie.getValue() + ", domain=" + cookie.getDomain() + ", path=" + cookie.getPath() + ", " + cookie.getMaxAge() + "]";
	}

	/**
	 * www.abc.com -> .abc.com abc.com -> .abc.com loclhost -> localhost
	 * 
	 * @param request
	 * @return
	 */
	public String getDomain(HttpServletRequest request) {
		String serverName = request.getServerName();
		return getDomainIn(serverName);
	}

	private String getDomainIn(String serverName) {
		int count = StringUtils.countMatches(serverName, ".");
		boolean isDigit = (count == 3) ? isDigit(serverName) : false;
		if (isDigit) {
			// 比如: 192.168.1.1
			return null;
		}
		if (count >= 1) {
			// newxtc.com -> .newxtc.com
			return "." + serverName;
		} else {
			// 比如: localhost
			return serverName;
		}
	}

	private final static String s1 = "0123456789";

	private static boolean isDigit(String domain) {
		char cs;
		String ck = domain.replaceAll("\\.", "");
		for (int i = 0; ck != null && i < ck.length(); i++) {
			cs = ck.charAt(i);
			if (s1.indexOf(cs) < 0) {
				return false;
			}
		}
		return true;
	}

	public void testCase(String domain) {
		System.out.println("domain=" + domain + " isDigit=" + isDigit(domain) + " ret=" + getDomainIn(domain));
	}

	public static void main(String[] args) {
		CookieUtil nxtCookieUtil = new CookieUtil();
		nxtCookieUtil.testCase("192.168.1.1");
		nxtCookieUtil.testCase("fp.newxtc.com");
		nxtCookieUtil.testCase("www.newxtc.com");
		nxtCookieUtil.testCase("www.newxtc.com.cn");
		nxtCookieUtil.testCase("localhost");
	}
}
