package com.newxtc.fw.client.util;

import java.util.Enumeration;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.newxtc.NxtInit;

public class JsonParamUtil {

	private static final String[] ipNames = new String[] { "X-Real-IP", "x-forwarded-for", "HTTP_X_FORWARDED_FOR", "Proxy-Client-IP", "WL-Proxy-Client-IP", "HTTP_CLIENT_IP" };

	// tokenId + tokenKey + begin+ lastTime + lastSign
	public static final String cookToken = "NXT_CLIENT_TOKEN";
	public static final String cookFrontId = "NXT_FRONT_ID";
	public static final String cookNav = "NXT_CLIENT_NAV";
	private final static Logger logger = LoggerFactory.getLogger(JsonParamUtil.class);
	public static final char[] CODES = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' };
	private static CookieUtil cookieUtil = new CookieUtil();

	public static JSONObject getRequestJson(HttpServletRequest request) {
		JSONObject inputJson = new JSONObject();
		String nxtToken = cookieUtil.getCookie(request, cookToken);
		String nxtNav = cookieUtil.getCookie(request, cookNav);

		if (nxtToken != null && nxtToken.length() >= 64) {
			inputJson.put("nxtToken", nxtToken);
		} else {
			logger.error("getRequestJson() nxtToken=" + nxtToken);
			return null;
		}

		if (nxtNav != null) {
			inputJson.put("nxtNav", nxtNav);
		} else {
			logger.error("getRequestJson() nxtNav=" + nxtNav);
		}
		return inputJson;
	}

	public static Map<String, Object> getParamByJson(JSONObject jsonObj) {
		Map<String, Object> paramMap = new LinkedHashMap<String, Object>();
		if (jsonObj != null) {
			String clientIp = jsonObj.getString("clientIp");
			String nxtToken = jsonObj.getString("nxtToken");
			String nxtNav = jsonObj.getString("nxtNav");
			String openid = jsonObj.getString("wx_open_id");
			String sysInfo = jsonObj.getString("wx_sys_info");
			if (clientIp != null && !"".equals(clientIp)) {
				paramMap.put("clientIp", clientIp);
			}
			if (nxtToken != null && nxtToken.length() >= 64) {
				paramMap.put("nxtToken", nxtToken);
			} else {
				logger.error("getParamByJson() nxtToken=" + nxtToken);
				return null;
			}
			if (nxtNav != null && !"".equals(nxtNav)) {
				paramMap.put("nxtNav", nxtNav);
			}
			if (openid != null && sysInfo != null) {
				paramMap.put("wx_open_id", openid);
				paramMap.put("wx_sys_info", sysInfo);
			}
		}
		return paramMap;
	}

	public static String getIpJson(HttpServletRequest request) {
		JSONObject ipJson = new JSONObject(true);
		String ip = null;
		for (String ipName : ipNames) {
			ip = request.getHeader(ipName);
			if (ip != null) {
				if (ip.contains(",")) {
					ip = ip.split(",")[0];
				}
				if ("0:0:0:0:0:0:0:1".equals(ip)) {
					ip = "127.0.0.1";
				}
				if (ip.contains(".")) {
					ipJson.put(ipName, ip);
				}
			}

		}
		ip = request.getRemoteAddr();
		if ("0:0:0:0:0:0:0:1".equals(ip)) {
			ip = "127.0.0.1";
		}
		boolean isLocal = "127.0.0.1".equals(ip);
		if (!isLocal || isLocal && ipJson.size() < 1) {
			ipJson.put("RemoteAddr", ip);
		}
		return ipJson.toJSONString();
	}

	public static String getReferrer(HttpServletRequest request) {
		Enumeration<String> headerNames = request.getHeaderNames();
		String lName = null, name = null, value = null;
		while (headerNames.hasMoreElements()) {
			name = headerNames.nextElement();
			lName = name.toLowerCase();
			if ("referrer".equals(lName) || "referer".equals(lName)) {
				value = request.getHeader(name);
				if (value != null && value.length() > 0) {
					return value;
				}
			}
		}
		return null;
	}

	public static Map<String, Object> getSignParam(Map<String, Object> paramMap) {
		String merchantCode = NxtInit.getInstance().getIni("merchantCode");
		String secKey = NxtInit.getInstance().getIni("secKey");
		if (merchantCode == null || secKey == null) {
			logger.error("getSignParam() merchantCode=" + merchantCode + ",secKey=" + secKey);
			return null;
		}
		Long time = System.currentTimeMillis();
		StringBuffer source = new StringBuffer(merchantCode + secKey + time);
		Object objToken = paramMap.get("nxtToken");
		if (objToken != null) {
			source.append(objToken);
		}
		Object objPhone = paramMap.get("phone");
		if (objPhone != null) {
			source.append(objPhone);
		}
		String sign = MD5Util.getMD5Str(source.toString());
		paramMap.put("MERCHANT_CODE", merchantCode);
		paramMap.put("time", time);
		paramMap.put("nxtSign", sign);
		return paramMap;
	}
}
