package com.newxtc.fw.client.util;

import java.util.LinkedHashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.newxtc.NxtInit;

public class WxAppUtil {
	private final static Logger logger = LoggerFactory.getLogger(WxAppUtil.class);

	/**
	 * 
	 * @param jsCode
	 * @param sysInfo
	 * @param clientIp
	 * @return
	 */
	public static String getWxOpenId(String jsCode, String sysInfo, String timezoneOffset, String clientIp) {
		JSONObject ret = new JSONObject();
		try {
			Map<String, Object> paramsMap = new LinkedHashMap<>();
			String wxAppid = NxtInit.getInstance().getIni("wxAppid");
			String wxSecret = NxtInit.getInstance().getIni("wxSecret");
			paramsMap.put("appid", wxAppid);
			paramsMap.put("secret", wxSecret);
			paramsMap.put("grant_type", "authorization_code");
			paramsMap.put("js_code", jsCode);
			String jsonResp = HttpClient.execClient("wxUrl", "", paramsMap, 500);
			JSONObject jsonObj = (jsonResp != null && jsonResp.contains("{") && jsonResp.contains("}")) ? JSONObject.parseObject(jsonResp) : null;
			String openid = (jsonObj != null) ? jsonObj.getString("openid") : null;
			if (openid != null) {
				JSONObject sysInfoJson = null;
				if (sysInfo != null && sysInfo.contains("{") && sysInfo.contains("}")) {
					sysInfoJson = JSONObject.parseObject(sysInfo);
				} else {
					sysInfoJson = new JSONObject();
				}
				if (clientIp != null) {
					sysInfoJson.put("clientIp", clientIp);
				}
				if (timezoneOffset != null) {
					sysInfoJson.put("timezoneOffset", timezoneOffset);
				}
				String retSysInfo = sysInfoJson.toJSONString();
				String source = wxAppid + wxSecret + openid + retSysInfo;
				String sign = MD5Util.getMD5Str(source);
				ret.put("openid", openid);
				ret.put("sysInfo", retSysInfo);
				ret.put("js_code", jsCode);
				ret.put("sign", sign);
				logger.debug("getWxOpenId() js_code:" + jsCode + " -> openid:" + openid);
			} else {
				logger.error("getWxOpenId() jsonResp=" + jsonResp + ",clientIp=" + clientIp);
			}
		} catch (Exception e) {
			logger.error("getWxOpenId() " + e.toString());
		}
		return ret.toJSONString();
	}

	/**
	 * 
	 * @param nxtData
	 * @return
	 */

	public static JSONObject getWxJson(String nxtData) {
		JSONObject jsonObj = new JSONObject();
		String merchantCode = NxtInit.getInstance().getIni("merchantCode");
		String secKey = NxtInit.getInstance().getIni("secKey");
		JSONObject jsonData = (nxtData != null && nxtData.contains("{") && nxtData.contains("}")) ? JSONObject.parseObject(nxtData) : null;
		if (jsonData != null) {
			String openid = jsonData.getString("openid");
			String sysInfo = jsonData.getString("sysInfo");
			String jsCode = jsonData.getString("js_code");
			String wxSign = jsonData.getString("sign");

			// 如果本地配置获取微信open，则验证签名
			String appid = NxtInit.getInstance().getIni("wxAppid");
			String secret = NxtInit.getInstance().getIni("wxSecret");
			boolean verify = true;
			if (appid != null && secret != null) {
				String source = appid + secret + openid + sysInfo;
				String vSign = MD5Util.getMD5Str(source);
				verify = (wxSign != null && wxSign.equals(vSign));
			}
			if (verify) {
				String tokenId = jsCode;
				if (jsCode.length() != 32) {
					tokenId = MD5Util.getMD5Str(jsCode);
				}
				String sign = MD5Util.getMD5Str(merchantCode + tokenId + secKey);
				String nxtToken = tokenId + sign;
				jsonObj.put("nxtToken", nxtToken);
				jsonObj.put("wx_open_id", openid);
				jsonObj.put("wx_sys_info", sysInfo);
			} else {
				logger.error("getWxJson() verify=" + verify + ",appid=" + appid + ",wxSign=" + wxSign);
			}
		}
		return jsonObj;
	}
}
